{-----------------------------------------------------------------------------
 Package:        MD - CP Gateway Service
 Developer(s):   Brian Juergensmeyer
 Date Created:   13-Oct-2006
 Site Name:      Hines OIFO
 Description:    TCPNotifier will receive incoming XML format messages from VistA.
                 These messages will generally either notify CP that there is a
                 message waiting for processing, or, ultimately, that a query against
                 CliO data has been generated.
 History:
   $Header: /CP Modernization/CliO/source/CP Gateway/uCPNotifier.pas 13    6/08/09 8:23a Zzzzzzjuergb $
   $History: uCPNotifier.pas $
 * 
 * *****************  Version 13  *****************
 * User: Zzzzzzjuergb Date: 6/08/09    Time: 8:23a
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * 
 * *****************  Version 12  *****************
 * User: Zzzzzzjuergb Date: 5/28/09    Time: 10:52a
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Build 275
 * 
 * *****************  Version 11  *****************
 * User: Zzzzzzjuergb Date: 3/30/09    Time: 10:07a
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Fixed error in timer value (was dividing time by 100 instead of
 * multiplying).
 * 
 * *****************  Version 10  *****************
 * User: Zzzzzzjuergb Date: 12/24/08   Time: 1:42p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * 
 * *****************  Version 9  *****************
 * User: Zzzzzzpetitd Date: 10/26/08   Time: 8:18p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 *
 * *****************  Version 8  *****************
 * User: Zzzzzzpetitd Date: 9/05/08    Time: 10:09p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Gateway work!
 *
 * *****************  Version 7  *****************
 * User: Zzzzzzpetitd Date: 9/04/08    Time: 12:42p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 *
 * *****************  Version 6  *****************
 * User: Zzzzzzjuergb Date: 11/19/07   Time: 8:40a
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Fix St. Louis problems with settings persistance.
 *
 * *****************  Version 5  *****************
 * User: Zzzzzzjuergb Date: 10/30/07   Time: 10:20a
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 *
 * *****************  Version 4  *****************
 * User: Zzzzzzjuergb Date: 10/27/07   Time: 7:01p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 *
 * *****************  Version 3  *****************
 * User: Zzzzzzjuergb Date: 10/24/07   Time: 9:26a
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 *
 * *****************  Version 2  *****************
 * User: Zzzzzzjuergb Date: 9/14/07    Time: 2:24p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Build 257
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzjuergb Date: 10/13/06   Time: 10:55a
 * Created in $/CP Modernization/CliO/source/CP Gateway
 * Initial checkin
-----------------------------------------------------------------------------}
unit uCPNotifier;

interface

uses
  Classes,
  SysUtils,
  Sockets,
  Contnrs,
  uCPMessage,
  oInterfaces,
  oGateway,
  ExtCtrls,
  oCPTypes;

type
  TCPNotifyEvent = procedure(Sender: TObject; AMessage: TCPMessage) of object;
  TCPNotifier = class
  private
    FTimer: TTimer;
    FLogger: ICPGatewayLogger;
    FGatewayConfig: ICPGatewayConfig;
    FDBConnection: IDBConnection;
    FOnCPNotify: TCPNotifyEvent;
    FOnCPReload: TNotifyEvent;
    FListener: TTcpServer;
    FRetrievalType: TDataRetrievalType;
    FPollingInterval: Integer;
    procedure DoPoll;
    procedure ConnectBroker;
    procedure DisconnectBroker;
{$IFDEF TEST}
  public
{$ENDIF}
    procedure ListenerAccept(Sender: TObject; ClientSocket: TCustomIpClient);
{$IFNDEF TEST}
  protected
{$ENDIF}
    procedure DoCPNotify(AMessage: TCPMessage); dynamic;
    procedure SetPort(const Value: Word);
    function GetPort: Word;
    function GetActive: boolean;
    procedure SetActive(const Value: boolean);
    function GetRetrievalType: TDataRetrievalType;
    procedure SetRetrievaltype(const Value: TDataRetrievalType);
    function GetPollingInterval: Integer;
    procedure SetPollingInterval(const Value: Integer);
    procedure FTimerOnTimer(Sender: TObject);
{$IFNDEF TEST}
  public
{$ENDIF}
    constructor Create(AGatewayConfig: ICPGatewayConfig; Logger: ICPGatewayLogger);
    destructor Destroy; override;
{$IFNDEF TEST}
  published
{$ENDIF}
    property OnCPNotify: TCPNotifyEvent read FOnCPNotify write FOnCPNotify;
    property OnCPReload: TNotifyEvent read FOnCPReload write FOnCPReload;
    property Port: Word read GetPort write SetPort;
    property Active: boolean read GetActive write SetActive;
    property RetrievalType: TDataRetrievalType read GetRetrievalType write SetRetrievalType;
    property PollingInterval: integer read GetPollingInterval write SetPollingInterval;
  end;

implementation

uses
  XMLDoc,
  XMLIntf,
  SyncObjs,
  Windows,
  Dialogs,
  ActiveX,
  Registry,
  Hash,
  TRPCB;

const
  HL7MSG_XML = '<?xml version="1.0" encoding="utf-8"?>' + #13#10 +
    '<message type="hl7">' + #13#10 +
    '  <id>' + #13#10 +
    '    %S' + #13#10 +
    '  </id>' + #13#10 +
    '  <text/>' + #13#10 +
    '</message>';

  //Registry information.
  REG_KEY = '\SOFTWARE\Vista\Clinical Procedures\CPGateway';
  CONTEXT = 'MDCP GATEWAY CONTEXT';

var
  CritSec: TCriticalSection;

  { TCPNotifier }

procedure TCPNotifier.ConnectBroker;
var
  ServerIP: string;
  Port: string;
  AccessCode: string;
  VerifyCode: string;
begin
  {Lets see if we can get a connection}
  if not FDBConnection.IsConnected then
    begin
      try

        ServerIP := FGatewayConfig.getBrokerServerIP;
        Port := IntToStr(FGatewayConfig.getBrokerServerPort);
        AccessCode := FGatewayConfig.getAccessCode;
        VerifyCode := FGatewayConfig.getVerifyCode;

        FLogger.Log('Attempting to connect the RPC Broker', llDetail);
        { DONE 1 -oBrian Juergensmeyer -cConfiguration :
          Switch broker server, listener port, access and verify codes to the local
          registry.  Note that we'll need to encrypt the access and verify codes. }

        FDBConnection.Connect([ServerIP, Port, AccessCode + ';' + VerifyCode, CONTEXT]);

        FLogger.Log(Format(
          'RPC Broker connection established: Server: %s, Port: %s',
          [ServerIP, Port]), llDetail);
      except
        on E: EBrokerError do
          begin
            FLogger.Log(Format(
              'RPC Broker connection failed: Server: %s, Port: %s, Error: %s',
              [ServerIP, Port, E.Message]), llError);
            FDBConnection.Disconnect;
          end;
        on E: Exception do
          begin
            FLogger.Log(Format('General error occurred: %s', [E.Message]), llError);
          end;
      end;
    end;
end;

constructor TCPNotifier.Create(AGatewayConfig: ICPGatewayConfig; Logger: ICPGatewayLogger);
begin
  FLogger := Logger;
  FGatewayConfig := AGatewayConfig;
  FDBConnection := FGatewayConfig.getDBConnection;
  FListener := TTcpServer.Create(nil);
  FListener.OnAccept := ListenerAccept;
  FListener.Active := false;
end;

destructor TCPNotifier.Destroy;
begin
  FListener.Active := false;
  FreeAndNil(FListener);
  inherited;
end;

procedure TCPNotifier.DisconnectBroker;
begin

end;

procedure TCPNotifier.DoCPNotify(AMessage: TCPMessage);
begin
  FLogger.Log('In CP notifier event publisher.', llDetail);

  if Assigned(FOnCPNotify) then
    FOnCPNotify(Self, AMessage);
end;

procedure TCPNotifier.DoPoll;
var
  Doc: IXMLDocument;
  Items: IXMLNode;
  Msg: TCPMessage;
  MsgID: string;
  I: Integer;
  MessageCount: integer;
begin
  CoInitialize(nil);
  try

    FLogger.Log('In DoPoll', llDetail);
    Doc := NewXMLDocument;

    repeat
    begin
      MessageCount := 0;
      FLogger.Log('Getting list of messages to process', llDetail);
      ConnectBroker;
      FDBConnection.ExecQuery('GetAPMessages', [], Doc);
      Doc.Active := True;
      FLogger.Log('Message list retrieved.', llDetail);

      Items := Doc.ChildNodes.FindNode('RESULTS');
      for I := 0 to Items.ChildNodes.Count - 1 do
      begin
        if Items.ChildNodes[I].NodeType = ntComment then
          Continue;

        inc(MessageCount);

        MsgID := Items.ChildNodes[I].ChildNodes[0].NodeValue;

        Msg := GenerateMessage(format(HL7MSG_XML, [MsgID]), FLogger);
        if not assigned(Msg) then
          begin
            FLogger.Log('Msg unassigned in ListenerAccept.', llError);
          end;

        FLogger.Log(Format('CP gateway listener received message with ID: %s. Executing CP Notify Event.', [Msg.ID]), llDetail);
        try
          DoCPNotify(Msg);
        finally
          FreeAndNil(Msg);
          DisconnectBroker;
        end;
      end;

      Sleep(5000);
    end
    until MessageCount = 0;
    
  finally
    CoUninitialize;
  end;
end;

procedure TCPNotifier.FTimerOnTimer(Sender: TObject);
begin
  DoPoll;
end;

function TCPNotifier.GetActive: boolean;
begin
  result := FListener.Active;
end;

function TCPNotifier.GetPollingInterval: Integer;
begin
  if FRetrievalType = drtPoll then
    result := FPollingInterval
  else
    result := 0;
end;

function TCPNotifier.GetPort: Word;
begin
  result := StrToIntDef(FListener.LocalPort, 0);
end;

function TCPNotifier.GetRetrievalType: TDataRetrievalType;
begin
  result := FRetrievalType;
end;

procedure TCPNotifier.ListenerAccept(Sender: TObject; ClientSocket: TCustomIpClient);
var
  ReceiveTxt: string;
  ReceiveDoc: IXMLDocument;
begin
  FLogger.Log('CP gateway listener accepting message.', llDetail);

  CritSec.Enter;
  try
    FLogger.Log('CP gateway listener accepting message.', llDetail);

    ReceiveTxt := ClientSocket.Receiveln;

    FLogger.Log(Format('CP gateway listener received text: %s', [ReceiveTxt]), llDetail);

    CoInitialize(nil);
    try
      ReceiveDoc := NewXMLDocument;
      ReceiveDoc.XML.Text := ReceiveTxt;
      ReceiveDoc.Active := True;
      if ReceiveDoc.DocumentElement.ChildValues['id'] = 'RELOAD' then
        begin
          FLogger.Log('Request for terminology reload received', llDetail);
          if Assigned(FOnCPReload) then
            FOnCPReload(nil);
        end
      else
        begin
          FLogger.Log('Log ID: ' + ReceiveDoc.DocumentElement.ChildValues['id'] + ' received.', llDetail);
          DoPoll;
        end;
    except
      on E: Exception do
        FLogger.Log(Format('Invalid Message Received: %s - Error: %s', [ReceiveTxt, E.Message]), llDetail);
    end;
    ReceiveDoc := nil;
    CoUninitialize;
  finally
    CritSec.Leave;
  end;
end;

procedure TCPNotifier.SetActive(const Value: boolean);
begin
  if not Assigned(FListener) then
    exit;

  if Value <> FListener.Active then
    begin
      FListener.Active := Value;
      if Value then
        begin
          FLogger.Log('CP Notifier performing initial poll for messages. ', llImportant);
          DoPoll;
          FLogger.Log('CP Notifier activated listening on port ' + FListener.LocalPort, llImportant);
        end
      else
        begin
          FLogger.Log('CP Notifier inactivated.', llImportant);
        end;

      if Value then
        if FRetrievalType = drtNotify then
          begin
            FLogger.Log('CP Notifier in VistA Notify mode.', llImportant);
            FListener.Active := Value;
          end
        else
          begin
            FLogger.Log(format('CP Notifier in polling mode.  Polling every %d seconds.', [FTimer.Interval div 1000]), llImportant);
            FTimer.Enabled := Value;
          end;
    end;
end;

procedure TCPNotifier.SetPollingInterval(const Value: Integer);
begin
  if FRetrievalType = drtPoll then
    begin
      {
      Time is store in config file in 10ths of a second.  The timer works on
      thousandths of a second.  Hence the multiplication below.
      }
      FPollingInterval := Value;
      FTimer.Interval := Value * 100;
    end;
end;

procedure TCPNotifier.SetPort(const Value: Word);
begin
  FListener.LocalPort := IntToStr(Value);
  if Assigned(FLogger) then
    FLogger.Log(Format('CP Notifier port set to %d.', [Value]), llImportant);
end;

procedure TCPNotifier.SetRetrievaltype(const Value: TDataRetrievalType);
begin
  if Value <> FRetrievalType then
    begin
      FRetrievalType := Value;

      if FRetrievalType = drtPoll then
        begin
          if not Assigned(FTimer) then
            begin
              FTimer := TTimer.Create(nil);
              FTimer.Interval := FPollingInterval;
              FTimer.OnTimer := FTimerOnTimer;
            end;
        end
      else
        begin
          FreeAndNil(FTimer);
        end;
    end;
end;

initialization
  CritSec := TCriticalSection.Create;

finalization;
  FreeAndNil(CritSec);

end.

