unit fGlobalUpdate;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ActnList,
  StdCtrls,
  ComCtrls;

type
  TfrmGlobalUpdate = class(TForm)
    Button1: TButton;
    Button2: TButton;
    lvEntries: TListView;
    cbxTermType: TComboBox;
    cbxFrom: TComboBox;
    cbxTo: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    acList: TActionList;
    acApply: TAction;
    acSearch: TAction;
    acLoadTerms: TAction;
    Button3: TButton;
    Label4: TLabel;
    procedure acListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure acSearchExecute(Sender: TObject);
    procedure acLoadTermsExecute(Sender: TObject);
    procedure SearchCriteriaHasChanged(Sender: TObject);
    procedure acApplyExecute(Sender: TObject);
  end;

procedure PerformGlobalUpdate;

implementation

uses
  uGlobalVars,
  iCPTerminology;

{$R *.dfm}

procedure PerformGlobalUpdate;
begin
  with TfrmGlobalUpdate.Create(Application.MainForm) do
    try
      lvEntries.Items.Clear;
      cbxTermType.Items.Clear;
      cbxFrom.Items.Clear;
      cbxTo.Items.Clear;
      MyTerminology.LoadTermTypes(cbxTermType.Items);
      ShowModal;
    finally
      Free;
    end;
end;

procedure TfrmGlobalUpdate.acApplyExecute(Sender: TObject);
begin
  Enabled := False;
  try
    ShowMessage('Applying...');
  finally
    lvEntries.Items.Clear;
  end;
  Enabled := True;
  if MessageDlg('Do you want to perform another batch update?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then
    ModalResult := mrOk;
end;

procedure TfrmGlobalUpdate.acListUpdate(Action: TBasicAction; var Handled: Boolean);
var
  aItemsSelected: integer;
begin
  acSearch.Enabled :=
    (cbxTermType.ItemIndex > -1) and
    (cbxFrom.ItemIndex > -1) and
    (cbxTo.ItemIndex > -1);

  aItemsSelected := 0;
  if lvEntries.Items.Count > 0 then
    with lvEntries.Items.GetEnumerator do
      while MoveNext do
        if Current.Checked then
          Inc(aItemsSelected);

  acApply.Enabled := (aItemsSelected > 0);
  {
  if (cbxTermType.ItemIndex > -1) and (cbxFrom.ItemIndex > -1) and (cbxTo.ItemIndex > -1) and (not fLoaded) then
    acLoadEntriesToUpdate.Execute
  else
    begin
      fLoaded := False;
      lvEntries.Items.Clear;
    end;
  }
  Handled := True;
end;

procedure TfrmGlobalUpdate.acSearchExecute(Sender: TObject);
var
  aFrom: ITerm;
  aFromID: string;
  aEntry: ITermMappingTableEntry;
begin
  lvEntries.Items.Clear;
  if cbxFrom.Items.Objects[cbxFrom.ItemIndex].GetInterface(ITerm, aFrom) then
    begin
      aFromID := aFrom.getID;
      with MyMappingTable.NewEnumerator do
        while MoveNext do
          if getCurrentEntry.getMappedTermID(TTermType(cbxTermType.ItemIndex)) = aFromID then
            GetCurrentEntry.AddToList(lvEntries);
          (*
          case TTermType(cbxTermType.ItemIndex) of
            ttObservation:
              if GetCurrentEntry.getObservationID = aFromID then
                GetCurrentEntry.AddToList(lvEntries);
            ttUnit:
              if GetCurrentEntry.getUnitID = aFromID then
                GetCurrentEntry.AddToList(lvEntries);
            ttMethod:
              if GetCurrentEntry.getMethodID = aFromID then
                GetCurrentEntry.AddToList(lvEntries);
              {
              ttPosition:
                if GetCurrentEntry.getPositionID = aFromID then
                  GetCurrentEntry.AddToList(lvEntries);
              }
            ttLocation:
              if GetCurrentEntry.getLocationID = aFromID then
                GetCurrentEntry.AddToList(lvEntries);
            ttQuality:
              if GetCurrentEntry.getQualityID = aFromID then
                GetCurrentEntry.AddToList(lvEntries);
            ttProduct:
              if GetCurrentEntry.getProductID = aFromID then
                GetCurrentEntry.AddToList(lvEntries);
              {
              ttNullFlavor:
                if GetCurrentEntry.getNullFlavorID = aFromID then
                  GetCurrentEntry.AddToList(lvEntries);
              }
              {
              ttTask:
                if GetCurrentEntry.getMethodID = aFromID then
                  GetCurrentEntry.AddToList(lvEntries);
              }
              {
              ttSchedule:
                if GetCurrentEntry.getScheduleID = aFromID then
                  GetCurrentEntry.AddToList(lvEntries);
              }
              {
              ttDataValue:
                if GetCurrentEntry.getDataValueID = aFromID then
                  GetCurrentEntry.AddToList(lvEntries);
              }
          end;
          *)
      with lvEntries.Items.GetEnumerator do
        while MoveNext do
          begin
            if Assigned(GetCurrent.Data) then
              if TObject(GetCurrent.Data).GetInterface(ITermMappingTableEntry, aEntry) then
                begin
                  GetCurrent.Checked := True;
                  GetCurrent.SubItems.Add(aEntry.getMappingDisplay);
                end;
          end;
    end;
end;

procedure TfrmGlobalUpdate.SearchCriteriaHasChanged(Sender: TObject);
begin
  lvEntries.Items.Clear;
end;

procedure TfrmGlobalUpdate.acLoadTermsExecute(Sender: TObject);
var
  aType: TTermType;
begin
  SearchCriteriaHasChanged(Sender);
  cbxFrom.Items.Clear;
  cbxTo.Items.Clear;
  if cbxTermType.ItemIndex > -1 then
    begin
      aType := TTermType(cbxTermType.ItemIndex);
      myTerminology.LoadList(cbxFrom.Items, aType);
      myTerminology.LoadList(cbxTo.Items, aType);
    end;
end;

end.

