unit fInstaller;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  uGlobalVars,
  XMLDoc,
  XMLIntf,
  uXML,
  ComCtrls,
  TRPCB,
  ActnList,
  ExtCtrls,
  iCPTerminology;

type
  TfrmInstaller = class(TForm)
    gbxStatus: TGroupBox;
    pBar: TProgressBar;
    btnInstall: TButton;
    btnClose: TButton;
    ActionList1: TActionList;
    acInstallTerminology: TAction;
    acLoadServer: TAction;
    lblMessage: TLabel;
    acConnect: TAction;
    acDisconnect: TAction;
    acUpdateTerms: TAction;
    Shape1: TShape;
    lblHeader: TLabel;
    Shape2: TShape;
    acLoadServerList: TAction;
    lblWelcome: TLabel;
    btnDisconnect: TButton;
    btnConnect: TButton;
    lblBrokerStatus: TLabel;
    acInstallMappingTable: TAction;
    procedure acInstallTerminologyExecute(Sender: TObject);
    procedure acConnectExecute(Sender: TObject);
    procedure acDisconnectExecute(Sender: TObject);
    procedure acUpdateTermsExecute(Sender: TObject);
    procedure ActionList1Update(Action: TBasicAction; var Handled: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure acInstallMappingTableExecute(Sender: TObject);
  private
    fBroker: TRPCBroker;
    fResults: TStringList;
    fSortedList: TStringList;
    fTermMappingTable: ITermMappingTable;
    procedure CallServer(aCommandName: string; aParams: array of variant);
    function TermExists(aTerm: ITerm): boolean;
    procedure InstallTerm(aTerm: ITerm);
    procedure InstallCDM(aTerm: ITerm);
    procedure InstallQualifiers(aTerm: ITerm);
    procedure InstallUnits(aTerm: ITerm);
    procedure InstallUnitConversions(aTerm: ITerm);
    procedure InstallChildTerms(aTerm: ITerm);
    procedure InstallMappingTable;
    procedure UpdateTerm(aTerm: ITerm);
    procedure setMessage(const aMessage: string);
  public
    { Public declarations }
  end;

procedure InstallTerminology;
procedure InstallMappingTable(aTermMappingTable: ITermMappingTable);

implementation

uses
  RPCConf1;

{$R *.dfm}

type
  TInstallType = (itTerminology, itMappingTable);

const
  BOOL2STR: array[Boolean] of string = ('0', '1');
  QUAL_NODES: array[0..5] of string = ('LOCATION_ID', 'METHOD_ID', 'POSITION_ID', 'QUALITY_ID', 'DATA_VALUE', 'PRODUCT_ID');

var
  InstallType: TInstallType;

procedure InstallTerminology;
begin
  with TfrmInstaller.Create(Application) do
    try
      Font := Screen.IconFont;
      lblHeader.Font.Style := [fsBold];
      lblBrokerStatus.Font.Style := [fsBold];
      InstallType := itTerminology;
      btnInstall.Action := acInstallTerminology;
      ShowModal;
    finally
      Free;
    end;
end;

procedure InstallMappingTable(aTermMappingTable: ITermMappingTable);
begin
  with TfrmInstaller.Create(Application) do
    try
      Font := Screen.IconFont;
      lblHeader.Font.Style := [fsBold];
      lblBrokerStatus.Font.Style := [fsBold];
      fTermMappingTable := aTermMappingTable;
      InstallType := itMappingTable;
      btnInstall.Action := acInstallMappingTable;
      ShowModal;
    finally
      Free;
    end;
end;

procedure TfrmInstaller.FormCreate(Sender: TObject);
begin
  fBroker := TRPCBroker.Create(Application);
  fResults := TStringList.Create;
end;

procedure TfrmInstaller.acConnectExecute(Sender: TObject);
var
  aSrvr: string;
  aPort: string;
begin
  setMessage('Connecting To VistA Server');
  try
    if GetServerInfo(aSrvr, aPort) <> mrOK then
      raise Exception.Create('Unable to determine Server/Port values');
    fBroker.Server := aSrvr;
    fBroker.ListenerPort := StrToInt(aPort);
    fBroker.Connected := True;

    if not fBroker.Connected then
      raise Exception.Create('Unable to connect to server');

    if not fBroker.CreateContext('MD CLIO') then
      raise Exception.Create('Unable to establish context ''MD CLIO''');

    setMessage('Broker connected.');
  except
    on E: Exception do
      raise Exception.CreateFmt('Error connecting: %s', [e.Message]);
  end;
end;

procedure TfrmInstaller.acDisconnectExecute(Sender: TObject);
begin
  try
    fBroker.Connected := False;
    setMessage('Broker disconnected.');
  except
    raise Exception.Create('Error disconnecting Broker');
  end;
end;

procedure TfrmInstaller.acInstallMappingTableExecute(Sender: TObject);
begin
  if MessageDlg('You are about to update files on this VistA system.' + #13 + #13 +
    'Are you absolutely sure you want to install?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    try
      Enabled := False;
      setMessage('Installing');
      gbxStatus.Visible := True;
      InstallMappingTable;
      acDisconnect.Execute;
      MessageDlg('Mapping Table Installed!', mtInformation, [mbOk], 0);
    except
      on E: Exception do
        MessageDlg('Error encountered installing Mapping Table.' + #13 + 'Error: ' + E.Message, mtError, [mbOk], 0);
    end;
  gbxStatus.Visible := False;
  Enabled := True;
end;

procedure TfrmInstaller.acInstallTerminologyExecute(Sender: TObject);
begin
  if MessageDlg('You are about to update files on this VistA system.' + #13 + #13 +
    'Are you absolutely sure you want to install?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    try
      Enabled := False;
      setMessage('Installing');
      gbxStatus.Visible := True;
      acConnect.Execute;
      acUpdateTerms.Execute;
      acDisconnect.Execute;
      MessageDlg('Terminology Installed!', mtInformation, [mbOk], 0);
    except
      on E: Exception do
        MessageDlg('Error encountered installing terminology.' + #13 + 'Error: ' + E.Message, mtError, [mbOk], 0);
    end;
  gbxStatus.Visible := False;
  Enabled := True;
end;

procedure TfrmInstaller.ActionList1Update(Action: TBasicAction; var Handled: Boolean);
begin
  acInstallTerminology.Enabled := fBroker.Connected;
  acInstallMappingTable.Enabled := fBroker.Connected;
  acConnect.Enabled := not fBroker.Connected;
  acDisconnect.Enabled := fBroker.Connected;
  if fBroker.Connected then
    lblBrokerStatus.Caption := Format('Broker Connected - Server: %s, Port: %d', [fBroker.Server, fBroker.ListenerPort])
  else
    lblBrokerStatus.Caption := 'Broker Disconnected';
  Handled := True;
end;

procedure TfrmInstaller.acUpdateTermsExecute(Sender: TObject);
var
  i: integer;
  aTerm: ITerm;
  aTermType: TTermType;
begin
  try
    fSortedList := TStringList.Create;
    MyTerminology.LoadList(fSortedList);
    fSortedList.Sorted := True;
    pBar.Min := 0;
    pBar.Max := MyTerminology.getCount;

    {First time through makes sure all the entries exist before we install CDM}
    SetMessage('Verifying existing Dictionary of Terms');
    for i := 0 to fSortedList.Count - 1 do
      if fSortedList.Objects[i].GetInterface(ITerm, aTerm) then
        begin
          pBar.Position := i;
          Application.ProcessMessages;
          if TermExists(aTerm) then
            UpdateTerm(aTerm)
          else
            InstallTerm(aTerm);
        end;

    {Now we have to remove the old CDM}
    SetMessage('Removing Old Clinical Data Model');
    CallServer('DeleteTermTypes', []);
    Application.ProcessMessages;
    CallServer('DeleteTermChildPairs', []);
    Application.ProcessMessages;
    CallServer('DeleteTermPairs', []);
    Application.ProcessMessages;
    CallServer('DeleteTermRangeChecks', []);
    Application.ProcessMessages;
    CallServer('DeleteTermUnitConversions', []);
    Application.ProcessMessages;
    CallServer('DeleteTermUnitPairs', []);

    SetMessage('Installing New Clinical Data Model');

    {Start with a fresh set of term types}
    for aTermType := ttObservation to High(TTermType) do
      CallServer('AddTermType',
        [Ord(aTermType), TERM_TYPE[aTermType], TERM_TYPE_XML[aTermType]]);

    Application.ProcessMessages;
    for i := 0 to fSortedList.Count - 1 do
      if fSortedList.Objects[i].GetInterface(ITerm, aTerm) then
        if aTerm.getActive then
          begin
            pBar.Position := i;
            Application.ProcessMessages;
            InstallCDM(aTerm);
          end;

  finally
    fSortedList.Clear;
    FreeAndNil(fSortedList);
  end;
end;

procedure TfrmInstaller.CallServer(aCommandName: string; aParams: array of variant);
var
  i: Integer;
begin
  try
    if not fBroker.Connected then {let's not bang on the thing endlessly}
      raise Exception.Create('RPC Broker is not connected');

    with fBroker do
      begin
        RemoteProcedure := 'MD CLIO';
        Param[0].Value := 'EXECUTE';
        Param[0].PType := literal;
        Param[1].Value := aCommandName;
        Param[1].PType := literal;
      end;

    {Load params into Param[2] if any}
    i := 0;
    if SizeOf(aParams) > 0 then
      begin
        fBroker.Param[2].PType := list;
        while i <= High(aParams) do
          begin
            if VarIsType(aParams[i], varDate) then
              fBroker.Param[2].Mult[IntToStr(i)] := FormatDateTime('YYYY-MM-DD hh:mm:ss', aParams[i]) //DateTimeToXMLString(aParams[i])
            else
              if VarIsType(aParams[i], varBoolean) then
                if aParams[i] then
                  fBroker.Param[2].Mult[IntToStr(i)] := '1'
                else
                  fBroker.Param[2].Mult[IntToStr(i)] := '0'
              else
                fBroker.Param[2].Mult[IntToStr(i)] := VarToStrDef(aParams[i], '');

            inc(i);
          end;
      end;

    try
      fResults.Clear;
      fBroker.lstCall(fResults);
    except
      on E: Exception do
        begin
          fBroker.Connected := False; {Force this thing to stop being STOOOPID!!!}
          raise Exception.Create('Broker communication failure.' + #13 + E.Message);
        end;
    end;

    if not (Length(fResults.Text) > 0) and (Copy(fResults.Text, 1, 2) <> '-1') then {Clear the error flag (-1^) out}
      raise Exception.Create(Copy(fResults.Text, 4, Length(fResults.Text)))
  except
    on E: Exception do
      begin
        fResults.Clear;
        raise Exception.Create(E.Message);
      end;
  end;
end;

procedure TfrmInstaller.setMessage(const aMessage: string);
begin
  lblMessage.Caption := Format('%s ...', [aMessage]);
  Application.ProcessMessages;
end;

function TfrmInstaller.TermExists(aTerm: ITerm): boolean;
var
  aXMLDoc: IXMLDocument;
begin
  try
    CallServer('GetTerms', [aTerm.getId]);
    aXMLDoc := NewXMLDocument;
    aXMLDoc.Active := True;
    aXMLDoc.XML.Text := fResults.Text;
    Result := (XMLDocumentFirstRecord(aXMLDoc) <> nil);
  finally
    aXMLDoc := nil;
  end;
end;

procedure TfrmInstaller.InstallCDM(aTerm: ITerm);
begin
  case aTerm.getTermType of
    ttObservation:
      begin
        InstallQualifiers(aTerm);
        InstallUnits(aTerm);
        InstallChildTerms(aTerm);
      end;
    ttUnit:
      begin
        InstallUnitConversions(aTerm);
      end;
  end;
end;

procedure TfrmInstaller.InstallChildTerms(aTerm: ITerm);
var
  aChild: ITermChildPair;
begin
  with aTerm.getComponentEnumerator(ctChildTerm) do
    while MoveNext do
      if GetCurrentComponent.QueryInterface(ITermChildPair, aChild) = 0 then
        try
          CallServer('AddTermChild', [
            aChild.getParentTerm.getID,
              aChild.getComponentOrder,
              aChild.getComponentTerm.getID,
              aChild.getValueUnitID,
              aChild.getValueType,
              Ord(aChild.getValueDelimiter),
              aChild.getValueStartPosition,
              aChild.getValueStopPosition,
              aChild.getComponentDescription]);
        except
          on E: Exception do
            MessageDlg('Error in installing Child Pair for ' + aTerm.getTerm + #13 + 'Message: ' + e.Message, mtError, [mbOk], 0);
        end;
end;

procedure TfrmInstaller.InstallMappingTable;
var
  aTermType: TTermType;
begin
  setMessage('Cleaning up existing entry');

  CallServer('DeleteMappingTableTerms', [fTermMappingTable.getID]);
  CallServer('DeleteMappingTable', [fTermMappingTable.getID]);
  setMessage('Installing New Table.');
  with fTermMappingTable do
    CallServer('AddMappingTable',
      [getID, getName, getHL7SendingApplication, BOOLEAN_TO_INT[getActive], getComment,
      getSourceID, getSourceApplication, getSourceVersion, BOOLEAN_TO_INT[getSourceTrusted]]);

  setMessage('Installing New Entries.');
  with fTermMappingTable.NewEnumerator do
    while MoveNext do
      with GetCurrentEntry do
        begin
          pBar.Position := GetPercentageDone;
          if getDoNotProcess then
            CallServer('AddMappingTableTerm',
              [fTermMappingTable.getID, getVendorKey, ttObservation, DO_NOT_PROCESS_ID])
          else
            for aTermType := Low(TTermType) to High(TTermType) do
              begin
                if getMappedTerm(aTermType) <> nil then
                  CallServer('AddMappingTableTerm',
                    [fTermMappingTable.getID, getVendorKey, aTermType, getMappedTermID(aTermType)]);
              end;
        end;
end;

procedure TfrmInstaller.InstallQualifiers(aTerm: ITerm);
var
  aQualifier: ITermQualifierPair;
begin
  with aTerm.getComponentEnumerator(ctQualifier) do
    while MoveNext do
      if GetCurrentComponent.QueryInterface(ITermQualifierPair, aQualifier) = 0 then
        try
          CallServer('AddTermPair',
            [aTerm.getId,
            aQualifier.getComponentOrder,
              aQualifier.getComponentTerm.getID,
              aQualifier.getRanking]);
        except
          on E: Exception do
            MessageDlg('Error in installing Qualifier for ' + aTerm.getTerm + #13 + 'Message: ' + e.Message, mtError, [mbOk], 0);
        end;
end;

procedure TfrmInstaller.InstallTerm(aTerm: ITerm);
begin
  with aTerm do
    CallServer('AddTerm',
      [getID, getTerm, getAbbreviation, getDisplayName, getTermType, getDataType]);
  UpdateTerm(aTerm);
end;

procedure TfrmInstaller.InstallUnitConversions(aTerm: ITerm);
var
  aConversion: ITermUnitConversion;
begin
  with aTerm.getComponentEnumerator(ctConversion) do
    while MoveNext do
      if GetCurrentComponent.QueryInterface(ITermUnitConversion, aConversion) = 0 then
        try
          CallServer('AddTermUnitConversion',
            [aTerm.getId,
            aConversion.getComponentTerm.getID,
              aConversion.getConversionOffsetPre,
              aConversion.getConversionOffsetPost,
              aConversion.getConversionFactor,
              aConversion.getDecimalPrecision,
              aConversion.getComponentDescription]);
        except
          on E: Exception do
            MessageDlg('Error in installing Qualifier for ' + aTerm.getTerm + #13 + 'Message: ' + e.Message, mtError, [mbOk], 0);
        end;
end;

procedure TfrmInstaller.InstallUnits(aTerm: ITerm);
var
  aUnitPair: ITermUnitPair;
begin
  with aTerm.getComponentEnumerator(ctUnit) do
    while MoveNext do
      if GetCurrentComponent.QueryInterface(ITermUnitPair, aUnitPair) = 0 then
        try
          CallServer('AddTermUnit',
            [aTerm.getId,
            aUnitPair.getComponentTerm.getID,
              aUnitPair.getMinValue,
              aUnitPair.getMaxValue,
              aUnitPair.getDecPrecision,
              aUnitPair.getReferenceLow,
              aUnitPair.getReferenceHigh]);
        except
          on E: Exception do
            MessageDlg('Error in installing Unit Pair for ' + aTerm.getTerm + #13 + 'Message: ' + e.Message, mtError, [mbOk], 0);
        end;
end;

procedure TfrmInstaller.UpdateTerm(aTerm: ITerm);
begin
  with aTerm do
    CallServer('UpdateTerm',
      [getID, getTerm, getAbbreviation, getDisplayName, getTermType, getDataType,
      getActive, getDescription, getHelpText, getBooleanValueTrue, getBooleanValueFalse,
        getMultiSelectPicklist]);
end;

end.

