unit oTerm;

interface

uses
  uXML,
  ComCtrls,
  XMLIntf,
  XMLDoc,
  SysUtils,
  Classes,
  iCPTerminology;

type
  TTerm = class(TInterfacedObject, ITerm)
  private
    fXMLNode: IXMLNode;
    fTerminology: ITerminologyController;
    fComponents: IInterfaceList;
    fChangeMade: boolean;

    fCreatedDateTime: TDateTime;
    fCreatedBy: string;
    fCreatedOnWkstn: string;

    fUpdatedDateTime: TDateTime;
    fUpdatedBy: string;
    fUpdatedOnWkstn: string;

    fID: string;
    fTerm: string;
    fAbbreviation: string;
    fDisplayName: string;
    fTermType: TTermType;
    fDataType: TDataType;
    fActive: boolean;
    fDescription: string;
    fHelpText: string;
    fBooleanValueTrue: string;
    fBooleanValueFalse: string;
    fMultiSelect: boolean;
    fSchedule: integer;
    fScheduleType: TScheduleType;
    fVUID: string;

    function getXMLNode: IXMLNode; stdcall;
    function getAbbreviation: string; stdcall;
    function getActive: boolean; stdcall;
    function getBooleanValueFalse: string; stdcall;
    function getBooleanValueTrue: string; stdcall;
    function getDataType: TDataType; stdcall;
    function getDateTimeCreated: TDateTime; stdcall;
    function getDescription: string; stdcall;
    function getDisplayName: string; stdcall;
    function getHelpText: string; stdcall;
    function getID: string; stdcall;
    function getMultiSelectPicklist: boolean; stdcall;
    function getSchedule: integer; stdcall;
    function getScheduleType: TScheduleType; stdcall;
    function getTerm: string; stdcall;
    function getSibling(aID: string): ITerm; stdcall;
    function getTermWithActiveFlag: string; stdcall;
    function getTermType: TTermType; stdcall;
    function getVUID: string; stdcall;
    function getListIndex(aList: TStrings): integer; stdcall;
    function getChangeMade: boolean; stdcall;
    function getComponentCount(aComponentType: TTermComponentType): integer; stdcall;
    function getComponentEnumerator(aComponentType: TTermComponentType): ITermComponentEnumerator; stdcall;
    function getIsValidDefinition: boolean; stdcall;

    procedure setAbbreviation(const aValue: string); stdcall;
    procedure setActive(const aValue: boolean); stdcall;
    procedure setBooleanValueFalse(const aValue: string); stdcall;
    procedure setBooleanValueTrue(const aValue: string); stdcall;
    procedure setDataType(const aValue: TDataType); stdcall;
    procedure setDescription(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setHelpText(const aValue: string); stdcall;
    procedure setID(const aValue: string); stdcall;
    procedure setMultiSelectPicklist(const aValue: boolean); stdcall;
    procedure setSchedule(const aValue: integer); stdcall;
    procedure setScheduleType(const aValue: TScheduleType); stdcall;
    procedure setTerm(const aValue: string); stdcall;
    procedure setTermType(const aValue: TTermType); stdcall;
    procedure setVUID(const aValue: string); stdcall;
    procedure setChangeMade(const aValue: boolean); stdcall;

    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode; aIncludeAbbv: boolean = False): TTreeNode; stdcall;
    function AddToListView(aListView: TListView): TListItem; stdcall;
    function AddToList(aList: TStrings): integer; stdcall;

    procedure LoadComponents(aList: TStrings; aUseDisplayName: boolean = False); overload; stdcall;
    procedure LoadComponents(aList: TStrings; aComponentType: TTermComponentType; aUseDisplayName: boolean = False); overload; stdcall;
    procedure LoadComponents(aList: TStrings; aQualifierType: TTermType; aUseDisplayName: boolean = False); overload; stdcall;
    procedure LoadComponentTerms(aList: TStrings; aTermType: TTermType); stdcall;
    procedure LoadErrors(aList: TStrings); stdcall;

    procedure Validate; stdcall;

    function NewComponent(aXMLNode: IXMLNode): ITermComponent; overload;
    function NewComponent(aComponentType: TTermComponentType; aComponentTerm: ITerm): ITermComponent; overload;

    function NewError(aType: TTermErrorType; aMessage: string): ITermError; stdcall;
    function NewToDoItem: ITermToDoItem; stdcall;

    procedure RemoveComponent(aComponent: ITermComponent); stdcall;
    procedure ClearComponents; stdcall;
    function ContainsTerm(aTerm: ITerm): boolean; stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;
  public
    constructor Create(aTerminology: ITerminologyController; aXMLNode: IXMLNode; aUser: string = ''; aWkstn: string = '');
    destructor Destroy; override;
  end;

implementation

uses
  oTermError,
  oTermQualifierPair,
  oTermUnitPair,
  oTermChildPair,
  oTermUnitConversion,
  oTermComponentEnumerator,
  Variants,
  oTermToDoItem;

const
  XML_ID = 'ID';
  XML_CREATED_DATE_TIME = 'CREATED_DATE_TIME';
  XML_CREATED_BY = 'CREATED_BY';
  XML_CREATED_ON_WKSTN = 'CREATED_ON_WKSTN';

  XML_UPDATED_DATE_TIME = 'UPDATED_DATE_TIME';
  XML_UPDATED_BY = 'UPDATED_BY';
  XML_UPDATED_ON_WKSTN = 'UPDATED_ON_WKSTN';

  XML_TERM = 'TERM';
  XML_ABBREVIATION = 'ABBREVIATION';
  XML_DISPLAY_NAME = 'DISPLAY_NAME';
  XML_TERM_TYPE = 'TERM_TYPE';
  XML_TERM_DATA_TYPE = 'DATA_TYPE';
  XML_ACTIVE = 'ACTIVE';
  XML_DESCRIPTION = 'DESCRIPTION';
  XML_HELP_TEXT = 'HELP_TEXT';
  XML_BOOL_VALUE_TRUE = 'BOOLEAN_VALUE_TRUE';
  XML_BOOL_VALUE_FALSE = 'BOOLEAN_VALUE_FALSE';
  XML_MULTI_SELECT = 'MULTI_SELECT';
  XML_SCHEDULE = 'SCHEDULE';
  XML_SCHEDULE_TYPE = 'SCHEDULE_TYPE';
  XML_VUID = 'VUID';

procedure TTerm.ClearComponents;
begin
  fComponents.Clear;
end;

function TTerm.ContainsTerm(aTerm: ITerm): boolean;
var
  aChild: ITermChildPair;
begin
  Result := False;
  with getComponentEnumerator(ctUnknown) do
    while MoveNext do
      begin
        if GetCurrentComponent.getComponentTerm = aTerm then
          Result := True;
        if GetCurrentComponent.QueryInterface(ITermChildPair, aChild) = 0 then
          if aChild.getValueUnit = aTerm then
            Result := True;
      end;
end;

constructor TTerm.Create(aTerminology: ITerminologyController; aXMLNode: IXMLNode; aUser: string = ''; aWkstn: string = '');
begin
  inherited Create;
  fXMLNode := aXMLNode;
  fChangeMade := False;

  if fXMLNode.ChildNodes.FindNode('ID') = nil then
    fXMLNode.ChildValues['ID'] := NewGUID;

  fID := fXMLNode.ChildValues[XML_ID];

  if fXMLNode.ChildNodes.FindNode(XML_CREATED_DATE_TIME) = nil then
    fXMLNode.ChildValues[XML_CREATED_DATE_TIME] := DateTimeToXMLString(Now);

  if fXMLNode.ChildNodes.FindNode(XML_CREATED_BY) = nil then
    if aUser <> '' then
      fXMLNode.ChildValues[XML_CREATED_BY] := GetUserFromWindows
    else
      fXMLNode.ChildValues[XML_CREATED_BY] := 'Unknown User';

  if fXMLNode.ChildNodes.FindNode(XML_CREATED_ON_WKSTN) = nil then
    if aWkstn <> '' then
      fXMLNode.ChildValues[XML_CREATED_ON_WKSTN] := GetComputerNetName
    else
      fXMLNode.ChildValues[XML_CREATED_ON_WKSTN] := 'Unknown Workstation';

  if fXMLNode.ChildNodes.FindNode(XML_UPDATED_DATE_TIME) = nil then
    fXMLNode.ChildValues[XML_UPDATED_DATE_TIME] := DateTimeToXMLString(Now);

  if fXMLNode.ChildNodes.FindNode(XML_UPDATED_BY) = nil then
    if aUser <> '' then
      fXMLNode.ChildValues[XML_UPDATED_BY] := GetUserFromWindows
    else
      fXMLNode.ChildValues[XML_UPDATED_BY] := 'Unknown User';

  if fXMLNode.ChildNodes.FindNode(XML_UPDATED_ON_WKSTN) = nil then
    if aWkstn <> '' then
      fXMLNode.ChildValues[XML_UPDATED_ON_WKSTN] := GetComputerNetName
    else
      fXMLNode.ChildValues[XML_UPDATED_ON_WKSTN] := 'Unknown Workstation';

  fTerminology := aTerminology;
  TInterfaceList.Create.GetInterface(IInterfaceList, fComponents);
end;

destructor TTerm.Destroy;
begin
  fComponents.Clear;
  fComponents := nil;
  fXMLNode := nil;
  inherited;
end;

{UI Functions}

function TTerm.AddToList(aList: TStrings): integer;
begin
  Result := aList.AddObject(getTermWithActiveFlag, Self);
end;

function TTerm.AddToListView(aListView: TListView): TListItem;
begin
  Result := aListView.Items.Add;
  Result.Caption := getTerm;
  Result.Data := Self;
end;

function TTerm.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode; aIncludeAbbv: boolean = False): TTreeNode;
begin
  if aIncludeAbbv then
    Result := aTreeView.Items.AddChildObject(aTreeNode, Format('%s (%s)', [getTerm, getAbbreviation]), self)
  else
    Result := aTreeView.Items.AddChildObject(aTreeNode, getTerm, self);
end;

procedure TTerm.LoadComponents(aList: TStrings; aUseDisplayName: boolean = False);
{
This is a debug function to load all components into a TStringList for display
}
var
  i: integer;
  aComp: ITermComponent;
begin
  aList.Clear;
  for i := 0 to fComponents.Count - 1 do
    if fComponents[i].QueryInterface(ITermComponent, aComp) = 0 then
      aList.Add('Term: ' + acomp.getComponentTerm.getTerm + ' Display Name: ' + aComp.getComponentDisplayName);
end;

procedure TTerm.LoadComponents(aList: TStrings; aComponentType: TTermComponentType; aUseDisplayName: boolean = False);
begin
  aList.Clear;
  with getComponentEnumerator(aComponentType) do
    while MoveNext do
      GetCurrentComponent.AddToList(aList, aUseDisplayName);
end;

procedure TTerm.LoadComponents(aList: TStrings; aQualifierType: TTermType; aUseDisplayName: boolean = False);
begin
  aList.Clear;
  with getComponentEnumerator(ctQualifier) do
    while MoveNext do
      if GetCurrentComponent.getComponentTerm.getTermType = aQualifierType then
        GetCurrentComponent.AddToList(aList, aUseDisplayName);
end;

procedure TTerm.LoadComponentTerms(aList: TStrings; aTermType: TTermType);
begin
  aList.Clear;
  with getComponentEnumerator(ctUnknown) do
    while MoveNext do
      if GetCurrentComponent.getComponentTerm <> nil then
        if GetCurrentComponent.getComponentTerm.getTermType = aTermType then
          GetCurrentComponent.getComponentTerm.AddToList(aList);
end;

procedure TTerm.LoadErrors(aList: TStrings);
begin
  if getComponentCount(ctError) > 0 then
    begin
      aList.Add(getTerm);
      aList.Add(Copy('-----------------------------------------------------------', 1, Length(getTerm)));
      with getComponentEnumerator(ctError) do
        while MoveNext do
          aList.Add(GetCurrentComponent.getDetailLine);
      aList.Add(' ');
    end;
end;

function TTerm.NewComponent(aXMLNode: IXMLNode): ITermComponent;
var
  aType: TTermComponentType;
  aComp: ITerm;
begin
  try
    aType := TTermComponentType(XMLValueAsInt(aXMLNode, 'COMPONENT_TYPE'));
    aComp := getSibling(XMLValueAsStr(aXMLNode, 'COMPONENT_TERM'));

    with NewComponent(aType, aComp) do
      begin
        Load(aXMLNode);
        QueryInterface(ITermComponent, Result);
      end;
  except
    Result := nil;
  end;
end;

function TTerm.NewComponent(aComponentType: TTermComponentType; aComponentTerm: ITerm): ITermComponent;
begin
  case aComponentType of
    ctQualifier: TTermQualifierPair.Create(Self, aComponentTerm).GetInterface(ITermQualifierPair, Result);
    ctUnit: TTermUnitPair.Create(Self, aComponentTerm).GetInterface(ITermUnitPair, Result);
    ctConversion: TTermUnitConversion.Create(Self, aComponentTerm).GetInterface(ITermUnitConversion, Result);
    ctChildTerm: TTermChildPair.Create(Self, aComponentTerm).GetInterface(ITermChildPair, Result);
    ctError: TTermError.Create(Self).GetInterface(ITermError, Result);
    ctToDoItem: TTermToDoItem.Create(Self).GetInterface(ITermToDoItem, Result);
  else
    Result := nil;
  end;
  if Assigned(Result) then
    begin
      fComponents.Add(Result);
      fChangeMade := True;
    end;
end;

function TTerm.NewError(aType: TTermErrorType; aMessage: string): ITermError;
begin
  if NewComponent(ctError, nil).QueryInterface(ITermError, Result) = 0 then
    begin
      Result.setErrorType(aType);
      Result.setErrorMessage(aMessage);
      Result.setDateTimeFound(Now);
    end
  else
    Result := nil;
end;

function TTerm.NewToDoItem: ITermToDoItem;
begin
  if NewComponent(ctToDoItem, nil).QueryInterface(ITermToDoItem, Result) = 0 then
    Result.setItemText('Hee Hee Hee...I can add it anywhere')
  else
    Result := nil;
end;

procedure TTerm.RemoveComponent(aComponent: ITermComponent);
var
  i: integer;
begin
  i := fComponents.IndexOf(aComponent);
  if i < 0 then
    Exit
  else
    begin
      fComponents.Delete(i);
      fChangeMade := true;
    end;
end;

{Private getters}

function TTerm.getAbbreviation: string;
begin
  Result := fAbbreviation;
end;

function TTerm.getActive: boolean;
begin
  Result := fActive;
end;

function TTerm.getBooleanValueFalse: string;
begin
  Result := fBooleanValueFalse;
end;

function TTerm.getBooleanValueTrue: string;
begin
  Result := fBooleanValueTrue;
end;

function TTerm.getChangeMade: boolean;
begin
  Result := fChangeMade;
end;

function TTerm.getComponentCount(aComponentType: TTermComponentType): integer;
var
  i: integer;
  aComp: ITermComponent;
begin
  Result := 0;
  for i := 0 to fComponents.Count - 1 do
    if fComponents[i].QueryInterface(ITermComponent, aComp) = 0 then
      if aComp.getComponentType = aComponentType then
        inc(Result);
end;

function TTerm.getComponentEnumerator(aComponentType: TTermComponentType): ITermComponentEnumerator;
begin
  TTermComponentEnumerator.Create(fComponents, aComponentType).GetInterface(ITermComponentEnumerator, Result);
end;

function TTerm.getDataType: TDataType;
begin
  Result := fDataType;
end;

function TTerm.getDateTimeCreated: TDateTime;
begin
  Result := fCreatedDateTime;
end;

function TTerm.getDescription: string;
begin
  Result := fDescription;
end;

function TTerm.getDisplayName: string;
begin
  Result := fDisplayName;
end;

function TTerm.getHelpText: string;
begin
  Result := fHelpText;
end;

function TTerm.getID: string;
begin
  Result := fID;
end;

function TTerm.getIsValidDefinition: boolean;
begin
  Result := getComponentCount(ctError) = 0;
end;

function TTerm.getListIndex(aList: TStrings): integer;
begin
  for Result := 0 to aList.Count - 1 do
    if Assigned(aList.Objects[Result]) then
      if aList.Objects[Result] = Self then
        Exit;
  Result := -1;
end;

function TTerm.getMultiSelectPicklist: boolean;
begin
  Result := fMultiSelect;
end;

function TTerm.getSchedule: integer;
begin
  Result := fSchedule;
end;

function TTerm.getScheduleType: TScheduleType;
begin
  Result := fScheduleType;
end;

function TTerm.getTerm: string;
begin
  Result := fTerm;
end;

function TTerm.getSibling(aID: string): ITerm;
begin
  Result := fTerminology.getTerm(aID);
end;

function TTerm.getTermType: TTermType;
begin
  Result := fTermType;
end;

function TTerm.getVuid: string;
begin
  Result := fVUID;
end;

function TTerm.getXMLNode: IXMLNode;
begin
  Result := fXMLNode;
end;

{Private setters}

procedure TTerm.setAbbreviation(const aValue: string);
begin
  fAbbreviation := aValue;
end;

procedure TTerm.setActive(const aValue: boolean);
begin
  fActive := aValue;
end;

procedure TTerm.setBooleanValueFalse(const aValue: string);
begin
  fBooleanValueFalse := aValue;
end;

procedure TTerm.setBooleanValueTrue(const aValue: string);
begin
  fBooleanValueTrue := aValue;
end;

procedure TTerm.setChangeMade(const aValue: boolean);
begin
  fChangeMade := aValue;
end;

procedure TTerm.setDataType(const aValue: TDataType);
begin
  fDataType := aValue;
end;

procedure TTerm.setDescription(const aValue: string);
begin
  fDescription := aValue;
end;

procedure TTerm.setDisplayName(const aValue: string);
begin
  fDisplayName := aValue;
end;

procedure TTerm.setHelpText(const aValue: string);
begin
  fHelpText := aValue;
end;

procedure TTerm.setID(const aValue: string);
begin
  fID := aValue;
end;

procedure TTerm.setMultiSelectPicklist(const aValue: boolean);
begin
  fMultiSelect := aValue;
end;

procedure TTerm.setSchedule(const aValue: integer);
begin
  fSchedule := aValue;
end;

procedure TTerm.setScheduleType(const aValue: TScheduleType);
begin
  fScheduleType := aValue;
end;

procedure TTerm.setTerm(const aValue: string);
begin
  fTerm := aValue;
end;

procedure TTerm.setTermType(const aValue: TTermType);
begin
  fTermType := aValue;
end;

procedure TTerm.setVUID(const aValue: string);
begin
  fVUID := aValue;
end;

function TTerm.getTermWithActiveFlag: string;
begin
  if getActive then
    Result := getTerm
  else
    Result := Format('%s (Inactive)', [getTerm]);
end;

{Validation and errors}

procedure TTerm.Validate;
var
  aMsg: string;
  i, j: integer;
  aTermComponent: ITermComponent;
begin
  for i := fComponents.Count - 1 downto 0 do
    if fComponents[i].QueryInterface(ITermComponent, aTermComponent) = 0 then
      if aTermComponent.getComponentType = ctError then
        fComponents.Delete(i);

  aTermComponent := nil;

  if fActive then
    try
      if not IsFileBoyStr('Term Name', getTerm, 1, 75, aMsg) then
        NewError(etDataValue, aMsg);

      if not IsFileBoyStr('Display Name', getDisplayName, 1, 75, aMsg) then
        NewError(etDataValue, aMsg);

      if not IsFileBoyStr('Abbreviation', getAbbreviation, 1, 10, aMsg) then
        NewError(etDataValue, aMsg);

      if not IsFileBoyStr('Description', getDescription, 1, 250, aMsg) then
        setDescription('No Description');

      if not IsFileBoyStr('Help-Text', getHelpText, 1, 250, aMsg) then
        setHelpText('No Help-Text');

      if getDataType = dtBoolean then
        begin
          if not IsFileBoyStr('Boolean Value True', getBooleanValueTrue, 1, 100, aMsg) then
            NewError(etDataValue, aMsg);

          if not IsFileBoyStr('Boolean Value False', getBooleanValueFalse, 1, 100, aMsg) then
            NewError(etDataValue, aMsg);
        end;

      {Check for at least one unit for numerics}
      if (getTermType = ttObservation) and (getDataType = dtNumeric) then
        begin
          j := 0;
          for i := 0 to fComponents.Count - 1 do
            if fComponents[i].QueryInterface(ITermComponent, aTermComponent) = 0 then
              if aTermComponent.getComponentType = ctUnit then
                inc(j);

          if j = 0 then
            NewError(etLogic, 'Numeric terms must have at least 1 unit assigned');
        end;

      {Picklist needs at least 2 data values}
      if (getTermType = ttObservation) and (getDataType = dtPickList) then
        begin
          j := 0;
          for i := 0 to fComponents.Count - 1 do
            if fComponents[i].QueryInterface(ITermComponent, aTermComponent) = 0 then
              if aTermComponent.getComponentType = ctQualifier then
                if aTermComponent.getComponentTerm.getTermType = ttDataValue then
                  inc(j);

          if j < 2 then
            NewError(etLogic, 'Picklist terms must have at least 2 data values assigned');
        end;

      for i := fComponents.Count - 1 downto 0 do {We might be removing some - go backwards}
        if fComponents[i].QueryInterface(ITermComponent, aTermComponent) = 0 then
          if aTermComponent.getComponentType in [ctQualifier, ctUnit, ctConversion, ctChildTerm] then
            begin
              if aTermComponent.getComponentTerm.getTermType in [ttConcept, ttNullFlavor] then
                RemoveComponent(aTermComponent)
              else
                aTermComponent.Validate;
            end;
    except
      NewError(etUnknown, 'Unknown Error in term.');
    end;
  fChangeMade := True;
end;

procedure TTerm.Load;
var
  aNode: IXMLNode;
begin
  try
    fCreatedDateTime := XMLValueAsDateTime(fXMLNode, XML_CREATED_DATE_TIME);
    fCreatedBy := XMLValueAsStr(fXMLNode, XML_CREATED_BY);
    fCreatedOnWkstn := XMLValueAsStr(fXMLNode, XML_CREATED_ON_WKSTN);

    fUpdatedDateTime := XMLValueAsDateTime(fXMLNode, XML_UPDATED_DATE_TIME);
    fUpdatedBy := XMLValueAsStr(fXMLNode, XML_UPDATED_BY);
    fUpdatedOnWkstn := XMLValueAsStr(fXMLNode, XML_UPDATED_ON_WKSTN);

    fID := XMLValueAsStr(fXMLNode, XML_ID);
    fTerm := XMLValueAsStr(fXMLNode, XML_TERM);
    fAbbreviation := XMLValueAsStr(fXMLNode, XML_ABBREVIATION);
    fDisplayName := XMLValueAsStr(fXMLNode, XML_DISPLAY_NAME);
    fTermType := TTermType(XMLValueAsInt(fXMLNode, XML_TERM_TYPE, 0));
    fDataType := TDataType(XMLValueAsInt(fXMLNode, XML_TERM_DATA_TYPE, 0));
    fActive := (XMLValueAsInt(fXMLNode, XML_ACTIVE, 0) = 1);
    fDescription := XMLValueAsStr(fXMLNode, XML_DESCRIPTION);
    fHelpText := XMLValueAsStr(fXMLNode, XML_HELP_TEXT);
    fBooleanValueTrue := XMLValueAsStr(fXMLNode, XML_BOOL_VALUE_TRUE);
    fBooleanValueFalse := XMLValueAsStr(fXMLNode, XML_BOOL_VALUE_FALSE);
    fMultiSelect := (XMLValueAsInt(fXMLNode, XML_MULTI_SELECT, 0) = 1);
    fSchedule := XMLValueAsInt(fXMLNode, XML_SCHEDULE, 60);
    fScheduleType := TScheduleType(XMLValueAsInt(fXMLNode, XML_SCHEDULE_TYPE, 0));
    fVUID := XMLValueAsStr(fXMLNode, XML_VUID);

    fComponents.Clear;

    if fXMLNode.ChildNodes.FindNode('COMPONENTS') <> nil then
      begin
        aNode := fXMLNode.ChildNodes.FindNode('COMPONENTS').ChildNodes.First;
        while Assigned(aNode) do
          try
            NewComponent(aNode);
          finally
            aNode := aNode.NextSibling;
          end;
      end;
    fChangeMade := False;
  except
    raise ETerminologyLoadError.Create('Error loading term');
  end;
end;

procedure TTerm.Save;
var
  i: integer;
  aNode: IXMLNode;
begin
  try
    fXMLNode.ChildNodes.Clear;

    fXMLNode.ChildValues[XML_CREATED_DATE_TIME] := DateTimeToXMLString(fCreatedDateTime);
    fXMLNode.ChildValues[XML_CREATED_BY] := fCreatedBy;
    fXMLNode.ChildValues[XML_CREATED_ON_WKSTN] := fCreatedOnWkstn;

    fXMLNode.ChildValues[XML_UPDATED_DATE_TIME] := DateTimeToXMLString(Now);
    fXMLNode.ChildValues[XML_UPDATED_BY] := GetUserFromWindows;
    fXMLNode.ChildValues[XML_UPDATED_ON_WKSTN] := GetComputerNetName;

    fXMLNode.ChildValues[XML_ID] := fID;
    fXMLNode.ChildValues[XML_TERM] := fTerm;
    fXMLNode.ChildValues[XML_ABBREVIATION] := fAbbreviation;
    fXMLNode.ChildValues[XML_DISPLAY_NAME] := fDisplayName;
    fXMLNode.ChildValues[XML_TERM_TYPE] := fTermType;
    fXMLNode.ChildValues[XML_TERM_DATA_TYPE] := fDataType;
    fXMLNode.ChildValues[XML_ACTIVE] := BOOLEAN_TO_INT[fActive];
    fXMLNode.ChildValues[XML_DESCRIPTION] := fDescription;
    fXMLNode.ChildValues[XML_HELP_TEXT] := fHelpText;

    if fDataType = dtBoolean then
      begin
        fXMLNode.ChildValues[XML_BOOL_VALUE_TRUE] := fBooleanValueTrue;
        fXMLNode.ChildValues[XML_BOOL_VALUE_FALSE] := fBooleanValueFalse;
      end;

    if fDataType = dtPicklist then
      fXMLNode.ChildValues[XML_MULTI_SELECT] := BOOLEAN_TO_INT[fMultiSelect];

    if fTermType = ttSchedule then
      begin
        fXMLNode.ChildValues[XML_SCHEDULE] := fSchedule;
        fXMLNode.ChildValues[XML_SCHEDULE_TYPE] := fScheduleType;
      end;

    if fVUID <> '' then
      fXMLNode.ChildValues[XML_VUID] := fVUID;

    if fComponents.Count > 0 then
      begin
        aNode := fXMLNode.AddChild('COMPONENTS');
        for i := 0 to fComponents.Count - 1 do
          (fComponents[i] as ITermComponent).Save(aNode.AddChild('RECORD'));
      end;

    fChangeMade := False;
  except
    raise ETerminologyLoadError.Create('Error saving Term');
  end;
end;

end.

