unit oTermToDoItem;

interface

uses
  ComCtrls,
  XMLIntf,
  SysUtils,
  Classes,
  oTermComponent,
  uXML,
  iCPTerminology;

type
  TTermToDoItem = class(TTermComponent, ITermToDoItem)
  private
    fID: string;
    fItemText: string;
    fStatus: TTermToDoStatus;
    fEnteredBy: string;
    fEnteredAtWorkstation: string;
    fDateTimeEntered: TDateTime;
    fDateTimeCompleted: TDateTime;
  public
    constructor Create(aTerm: ITerm);
    destructor Destroy; override;

    function getID: string; stdcall;
    function getItemText: string; stdcall;
    function getStatus: TTermToDoStatus; stdcall;
    function getEnteredBy: string; stdcall;
    function getEnteredAtWorkstation: string; stdcall;
    function getDateTimeEntered: TDateTime; stdcall;
    function getDateTimeCompleted: TDateTime; stdcall;

    procedure setItemText(const aValue: string); stdcall;
    procedure setStatus(const aValue: TTermToDoStatus); stdcall;
    procedure setDateTimeCompleted(const aValue: TDateTime); stdcall;

    procedure Load(aXMLNode: IXMLNode); override; stdcall;
    procedure Save(aXMLNode: IXMLNode); override; stdcall;

    function AddToListView(aListView: TListView): TListItem; override; stdcall;
  end;

implementation

const
  {Default XML Tags for the properties}
  XML_ID = 'ID';
  XML_DISPLAY_TEXT = 'DISPLAY_TEXT';
  XML_STATUS = 'STATUS';
  XML_ENTERED_BY = 'ENTERED_BY';
  XML_DATETIME_ENTERED = 'DATE_TIME_ENTERED';
  XML_DATETIME_COMPLETED = 'DATE_TIME_COMPLETED';
  XML_WORKSTATION = 'WORKSTATION';

  {Default values for the properties}
  DEFAULT_DISPLAY_TEXT = 'New ToDo Item';
  DEFAULT_STATUS = tdPending;

constructor TTermToDoItem.Create(aTerm: ITerm);
begin
  inherited Create(ctToDoItem, aTerm, aTerm);
  fID := NewGUID;
  fItemText := DEFAULT_DISPLAY_TEXT;
  fStatus := DEFAULT_STATUS;
  fEnteredBy := GetUserFromWindows;
  fEnteredAtWorkstation := GetComputerNetName;
  fDateTimeEntered := Now;
  fDateTimeCompleted := Now;
end;

destructor TTermToDoItem.Destroy;
begin
  inherited;
end;

function TTermToDoItem.AddToListView(aListView: TListView): TListItem;
begin
  Result := aListView.Items.Add;
  Result.Caption := DateTimeToXMLString(getDateTimeEntered);
  Result.SubItems.Add(TERM_TODO_STATUS[fStatus]);
  Result.SubItems.Add(getComponentDescription);
  Result.Data := Self;
end;

procedure TTermToDoItem.Load(aXMLNode: IXMLNode);
begin
  try
    inherited Load(aXMLNode);
    fID := XMLValueAsStr(aXMLNode, XML_ID);
    fItemText := XMLValueAsStr(aXMLNode, XML_DISPLAY_TEXT, DEFAULT_DISPLAY_TEXT);
    fStatus := TTermToDoStatus(XMLValueAsInt(aXMLNode, XML_STATUS, Ord(DEFAULT_STATUS)));
    fEnteredBy := XMLValueAsStr(aXMLNode, XML_ENTERED_BY, GetUserFromWindows);
    fEnteredAtWorkstation := XMLValueAsStr(aXMLNode, XML_WORKSTATION, GetComputerNetName);
    fDateTimeEntered := XMLValueAsDateTime(aXMLNode, XML_DATETIME_ENTERED);
    fDateTimeCompleted := XMLValueAsDateTime(aXMLNode, XML_DATETIME_COMPLETED);
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to load %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

procedure TTermToDoItem.Save(aXMLNode: IXMLNode);
begin
  try
    inherited Save(aXMLNode);
    aXMLNode.ChildValues[XML_ID] := fID;
    aXMLNode.ChildValues[XML_DISPLAY_TEXT] := fItemText;
    aXMLNode.ChildValues[XML_STATUS] := fStatus;
    aXMLNode.ChildValues[XML_ENTERED_BY] := fEnteredBy;
    aXMLNode.ChildValues[XML_WORKSTATION] := fEnteredAtWorkstation;
    aXMLNode.ChildValues[XML_DATETIME_ENTERED] := DateTimeToXMLString(fDateTimeEntered);
    aXMLNode.ChildValues[XML_DATETIME_COMPLETED] := DateTimeToXMLString(fDateTimeCompleted);
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to save %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

function TTermToDoItem.getDateTimeCompleted: TDateTime;
begin
  Result := fDateTimeCompleted;
end;

function TTermToDoItem.getDateTimeEntered: TDateTime;
begin
  Result := fDateTimeEntered;
end;

function TTermToDoItem.getItemText: string;
begin
  Result := fItemText;
end;

function TTermToDoItem.getEnteredAtWorkstation: string;
begin
  Result := fEnteredAtWorkstation;
end;

function TTermToDoItem.getEnteredBy: string;
begin
  Result := fEnteredBy;
end;

function TTermToDoItem.getID: string;
begin
  Result := fID;
end;

function TTermToDoItem.getStatus: TTermToDoStatus;
begin
  Result := fStatus;
end;

procedure TTermToDoItem.setDateTimeCompleted(const aValue: TDateTime);
begin
  fDateTimeCompleted := aValue;
end;

procedure TTermToDoItem.setItemText(const aValue: string);
begin
  fItemText := aValue;
end;

procedure TTermToDoItem.setStatus(const aValue: TTermToDoStatus);
begin
  fStatus := aValue;
end;

end.

